#include <alloc.h>
#include <string.h>
#include <stdio.h>
#include "tools.h"
#include "define.h"
#include "wwv.h"

struct
{
  char 	sNodeCall[10];
  char 	sLogger[7];
  char 	sForecast[81];
  char 	sDate[12];
  char 	sTime[5];
  int	iSFI;
  int	iA;
  int	iK;
} WwvSpot;

//--------------------------------------------------------------------------
//------ Ajouter un WWV dans le fichier WWV --------------------------------
//--------------------------------------------------------------------------
void WWV_add(char * NodeCall, char * Logger, char * Forecast, char * Date, char * Time, int SFI, int A, int K)
{
  FILE * fPtr;

  //Verifier la validite de certains champs
  if( SFI > 999 || A > 999 || K > 99 )
    return; //Ne pas aller plus loin

  //------ Ouvrir le fichier et placer le nouveau WWV  la fin -------------
  fPtr = fopen(WWV_FILE, "a+b");

  if( ! fPtr )	//Le fichier n'a pu tre ouvert
  {
    perror("fopen in WWV_add");
    return;
  }/*End IF*/

  //Limiter la longueur des champs
  TOOLS_maxLength(NodeCall, 9);
  TOOLS_removeSsid(Logger);
  TOOLS_maxLength(Logger, 6);
  TOOLS_maxLength(Forecast, 80);
  TOOLS_maxLength(Date, 11);
  TOOLS_maxLength(Time, 4);

  //Prparer tous les champs de l'enregistrement
  memset(&WwvSpot, 0, sizeof(WwvSpot));		//pour rendre le fichier plus clean
  strcpy(WwvSpot.sNodeCall, 	NodeCall);
  strcpy(WwvSpot.sLogger,   	Logger);
  strcpy(WwvSpot.sForecast,	Forecast);
  strcpy(WwvSpot.sDate,		Date);
  strcpy(WwvSpot.sTime,		Time);
  WwvSpot.iSFI	= SFI;
  WwvSpot.iA	= A;
  WwvSpot.iK	= K;

  //Enregistrer le WWV  la fin du fichier
  fwrite(&WwvSpot, sizeof(WwvSpot), 1, fPtr);

  fclose(fPtr);
}

//--------------------------------------------------------------------------
//------ Lire un fichier dans WWV.BIN --------------------------------------
//--------------------------------------------------------------------------
int WWV_get(int Pos, char * NodeCall, char * Logger, char * Forecast, char * Date, char * Time, int * SFI, int * A, int * K)
{
  FILE *fPtr;
  long	lOffset;

  //------ Ouvrir le fichier et placer le nouveau WWV  la fin -------------
  fPtr = fopen(WWV_FILE, "a+b");

  if( ! fPtr )	//Le fichier n'a pu tre ouvert
  {
    perror("fopen in WWV_get");
    return FALSE;
  }/*End IF*/

  //Se placer  la fin du fichier
  fseek(fPtr, 0L, SEEK_END);

  //Remonter de Pos enregistrements
  lOffset =  (long) Pos * (long) sizeof(WwvSpot);
  if( lOffset > ftell(fPtr) )
  {
    fclose( fPtr );
    return FALSE;
  }
  fseek(fPtr, -lOffset, SEEK_CUR);

  //Extraire les champs
  fread(&WwvSpot, sizeof(WwvSpot), 1, fPtr);
  strcpy(NodeCall, 	WwvSpot.sNodeCall);
  strcpy(Logger,   	WwvSpot.sLogger);
  strcpy(Forecast,	WwvSpot.sForecast);
  strcpy(Date,		WwvSpot.sDate);
  strcpy(Time,		WwvSpot.sTime);
  *SFI	= 	WwvSpot.iSFI;
  *A	= 	WwvSpot.iA;
  *K	= 	WwvSpot.iK;

  //Fermer le fichier et retourner
  fclose( fPtr );
  return TRUE;
}

/*---------------------------------------------------------------------------
  ------ Rechercher si un WWV a deja ete recu -------------------------------
  ---------------------------------------------------------------------------
  Retourne FALSE si un wwv a deja ete recu dans la derniere fenetre de 3
  heures */
int WWV_isAllowed(char * szDate, char * szTime)
{
	FILE   *fPtr;
	long	lOffset;

	/* Ouvrir le fichier */
	fPtr = fopen(WWV_FILE, "rb");

	if( ! fPtr )	/* Impossible d'ouvrir */
	{
		perror("fopen in WWV_isDupe");
		return FALSE;
	}

	/* Placer le pointeur de fichier NLAST spots en arriere */
	fseek(fPtr, 0L, SEEK_END);			/* Fin du fichier */
	lOffset =  (long) sizeof(WwvSpot);		/* Calcul offset */
	if( lOffset > ftell(fPtr) )
		fseek(fPtr, 0L, SEEK_SET);		/* Se placer en debut de fichier */
	else
		fseek(fPtr, -lOffset, SEEK_CUR);	/* Pointer sur le dernier enregistrement */

	/* Lecture enregistrement */
	if( ! fread(&WwvSpot, sizeof(WwvSpot), 1, fPtr) )
	{
		fclose(fPtr);
		return TRUE;	/* EOF */
	}

	fclose(fPtr);

	/* Tester */
	if( strcmp(szDate, WwvSpot.sDate) )
		return TRUE;	/* Date differente : OK */

	if( (atoi(szTime) / 300) == (atoi(WwvSpot.sTime) / 300) )
		return FALSE;	/* Meme fenetre horaire : rejeter */

	return TRUE;		/* Ok */
}